@@ -1,22 +1,21 @@
-import re, socket, threading, signal, json, errno, time
-from time import sleep
+import re, socket, errno
+from threading import Thread
+from signal import signal, SIGINT, SIGTERM
 from datetime import datetime
+from time import time
 from .BaseAPIRequest import BaseAPIRequest, APIMethod
-from .APIModel import APIJSONEncoder, APIModel
 from .http_request import http_request
-from .errno import *
+from .http_response import http_response
+from .error_no import *
 from .exception import *
+from .docs.docs import _docs
+from ._internals import _iuList, _parse_unicode_value
 from . import LITEAPI_SUPPORTED_REQUEST_METHODS
 
-RETURN_STATUS = lambda c : '{} {}'.format(c, strerror(c))
-RETURN_STATUS_OBJ = lambda c : {'code': c, 'message': strerror(c)}
-
-JSON_UTF8 = 'application/json; charset=utf-8'
-
 class liteapi:
     class __version:
         MAJOR = 0
-        MINOR = 4
+        MINOR = 5
         PATCH = 1
         
         def __str__(self):
@@ -36,14 +35,9 @@ class liteapi:
         self.__request = {}
         self.__config = liteapi.__defaultConfig.copy()
         self.__config.update(kwargs)
+        self.__fixedHeaders = _iuList()
         self.__init_socket()
-        @self.register('/info')
-        class infoClass (BaseAPIRequest):
-            def get(myself):
-                info = []
-                for regex in self._liteapi__request:
-                    info.append({'uri': self._liteapi__request[regex]._BaseAPIRequest__definition, 'methods': self._liteapi__request[regex]._BaseAPIRequest__methods_keys})
-                return info
+        self.__docs = _docs(self) if 'nodoc' not in kwargs or not kwargs['nodoc'] else None
     def __del__(self):
         self.close()
     def close(self):
@@ -58,12 +52,14 @@ class liteapi:
             self.__socket = None
     def run(self):
         self.__running = True
+        if(self.__docs):
+            self.__docs.build()
         print("""API Server started on URL:
         http{}://{}:{}""".format('s' if self.__ssl else '', self.__config['host'], self.__config['port']))
         while self.__running:
             try:
                 conn, addr = self.__socket.accept()
-                thread = threading.Thread(target=self.__handle_client, args=[conn, addr])
+                thread = Thread(target=self.__handle_client, args=[conn, addr])
                 thread.daemon = True
                 thread.start()
             except:
@@ -74,47 +70,74 @@ class liteapi:
                 raise RuntimeError('{} must be a subclass of BaseAPIRequest'.format((requestClass).__name__))
             hasQuery = uri.find('?')
             regex = uri[:hasQuery] if hasQuery > 0 else uri
-            requestClass._BaseAPIRequest__definition = re.sub(':(str|int|float)', '', regex)
+            definition = re.sub('{([^:}]+):(str|int|float|path)}', r'{\1}', regex)
+            requestClass._BaseAPIRequest__definition = definition
             if not requestClass._BaseAPIRequest__methods:
                 requestClass._BaseAPIRequest__methods = {}
             if not requestClass._BaseAPIRequest__methods_keys:
                 requestClass._BaseAPIRequest__methods_keys = []
-            requestClass._BaseAPIRequest__response = {'content-type': JSON_UTF8}
             requestClass._BaseAPIRequest__uriVars = {}
 
-            for methodnam in LITEAPI_SUPPORTED_REQUEST_METHODS:
-                if methodnam.lower() in dir(requestClass):
-                    requestClass._BaseAPIRequest__methods[methodnam] = method = APIMethod.create(methodFunc=getattr(requestClass, methodnam.lower()))
-                    setattr(requestClass, methodnam.lower(), method)
-                    requestClass._BaseAPIRequest__methods_keys.append(methodnam)
-            
-            ms = re.findall('\{(([^:}]+)(:(str|int|float))?)\}', regex)
+            ms = re.findall('\{(([^:}]+)(:(str|int|float|path))?)\}', regex)
             for m in ms:
-                if len(m) > 3 and m[3] in ['int', 'float']:
+                if len(m) > 3 and m[3] in ('int', 'float', 'path'):
                     if m[3] == 'int':
                         regex = regex.replace('{{{}}}'.format(m[0]), '([0-9]+)')
                         if m[1] not in requestClass._BaseAPIRequest__uriVars:
                             requestClass._BaseAPIRequest__uriVars[m[1]] = int
                     elif m[3] == 'float':
-                        regex = regex.replace('{{{}}}'.format(m[0]), r'([0-9]+\.[0-9]*|[0-9]*\.[0-9]+)')
+                        regex = regex.replace('{{{}}}'.format(m[0]), r'([0-9]+\.[0-9]*|[0-9]*\.[0-9]+|[0-9]+)')
                         if m[1] not in requestClass._BaseAPIRequest__uriVars:
                             requestClass._BaseAPIRequest__uriVars[m[1]] = float
+                    elif m[3] == 'path':
+                        regex = regex.replace(f'{{{m[0]}}}', '([^?&]+)')
+                        if m[1] not in requestClass._BaseAPIRequest__uriVars:
+                            requestClass._BaseAPIRequest__uriVars[m[1]] = str
                 else:
                     regex = regex.replace('{{{}}}'.format(m[0]), '([^/?&]+?)')
                     if m[1] not in requestClass._BaseAPIRequest__uriVars:
                         requestClass._BaseAPIRequest__uriVars[m[1]] = str
                 regex = regex.replace('{{{}}}'.format(m[0]), '([0-9]+)' if m[3] == 'int' else '([^/?&]+?)')
-            self.__request[regex] = requestClass
+            requestClass._BaseAPIRequest__regex = regex
+
+            for methodnam in LITEAPI_SUPPORTED_REQUEST_METHODS:
+                if methodnam.lower() in dir(requestClass):
+                    requestClass._BaseAPIRequest__methods[methodnam] = method = APIMethod.create(methodFunc=getattr(requestClass, methodnam.lower()))
+                    args = {}
+                    for arg in requestClass._BaseAPIRequest__uriVars:
+                        args[arg] = requestClass._BaseAPIRequest__uriVars[arg]
+                    args.update(method.args)
+                    method.args = args
+                    if len(method.responses) == 0:
+                        if isinstance(method.returnType, type) and issubclass(method.returnType, APIModel):
+                            method.responses.append((200, "Successful Response", method.returnType))
+                        else:
+                            method.responses.append((200, "Successful Response"))
+                        method.defaultResponse = True
+                    setattr(requestClass, methodnam.lower(), method)
+                    requestClass._BaseAPIRequest__methods_keys.append(methodnam)
+
+            self.__request[definition] = requestClass
+            if(requestClass._hasDoc):
+                print(f'API {requestClass._BaseAPIRequest__definition} is registered')
         return inner
+    def extend_supported_request_content_types(self, mimetype, parser_function, override = False, request_property = 'obj'):
+        http_request.extend_supported_content_types(mimetype, parser_function, override, request_property)
+    def extend_supported_response_content_types(self, mimetype, parser_function, override = False):
+        http_response.extend_supported_content_types(mimetype, parser_function, override)
+    def add_fixed_headers(self, name, *args):
+        self.__fixedHeaders.append(name)
+        for a in args:
+            self.__fixedHeaders.append(a)
     def __handle_client(self, sock, addr):
         BUFF_SIZE = 4096
         request_data = b''
-        response = 'HTTP/1.1 {}\r\ncontent-length: {}\r\n{}\r\n{}'
+        response_format = 'HTTP/1.1 {}\r\nContent-Length: {}\r\n{}\r\n'
         stime = 0
         while True:
             try:
                 if not stime:
-                    stime =  time.time()
+                    stime =  time()
                 part = sock.recv(BUFF_SIZE)
                 request_data += part
                 if len(part) < BUFF_SIZE:
@@ -122,82 +145,97 @@ class liteapi:
             except socket.error as e:
                 err = e.args[0]
                 if err in (errno.EAGAIN, errno.EWOULDBLOCK):
-                    stime =  time.time()
+                    stime =  time()
                     continue
         if not request_data:
             sock.close()
             return
-        request = http_request(request_data)
+        
+        try:
+            request = http_request(request_data)
+        except Exception as e:
+            sock.close()
+            return
+        
         try:
             if request.version != 'HTTP/1.1':
                 raise APIException(HTTP_VERSION_NOT_SUPPORTED)
             if request.method not in self.__supportedMethods:
                 raise APIException(METHOD_NOT_ALLOWED)
-            found, uriRegex, vars = False, None, {}
-            for uriRegex in self.__request:
-                ms = re.fullmatch(uriRegex, request.base_uri)
+            request._http_request__protocol = f'http{"s" if self.__ssl else ""}'
+            request._http_request__host = self.__config['host']
+            request._http_request__port = self.__config['port']
+            found, definition, vars = False, None, {}
+            for definition in self.__request:
+                ms = re.fullmatch(self.__request[definition]._BaseAPIRequest__regex, request.base_uri)
                 if ms:
                     found = True
-                    if request.method not in self.__request[uriRegex]._BaseAPIRequest__methods and (request.method != 'HEAD' or (request.method == 'HEAD' and 'GET' not in self.__request[uriRegex]._BaseAPIRequest__methods)):
+                    if request.method not in self.__request[definition]._BaseAPIRequest__methods and (request.method != 'HEAD' or (request.method == 'HEAD' and 'GET' not in self.__request[definition]._BaseAPIRequest__methods)):
                         raise APIException(NOT_IMPLEMENTED)
                     else:
-                        for i in range(len(self.__request[uriRegex]._BaseAPIRequest__uriVars)):
-                            variable = [k for k in self.__request[uriRegex]._BaseAPIRequest__uriVars.keys()][i]
-                            vars[variable] = self.__request[uriRegex]._BaseAPIRequest__uriVars[variable](ms[i + 1])
+                        var_keys = [k for k in self.__request[definition]._BaseAPIRequest__uriVars.keys()]
+                        for i in range(len(self.__request[definition]._BaseAPIRequest__uriVars)):
+                            vars[var_keys[i]] = _parse_unicode_value(self.__request[definition]._BaseAPIRequest__uriVars[var_keys[i]](ms[i + 1]))
                     break
             if not found:
                 raise APIException(NOT_FOUND)
             
-            response_code = RESPONSE_OK
-            response_status = RETURN_STATUS(response_code)
-            copyRequest = self.__request[uriRegex]()
-            copyRequest._BaseAPIRequest__request = request
-            copyRequest.client_address = addr[0]
-
             request.parseData()
-            
-            response_data_raw = copyRequest._BaseAPIRequest__methods[request.method if request.method != 'HEAD' else 'GET'](copyRequest, **vars)
 
-            if('json' in copyRequest.response['content-type'] and isinstance(response_data_raw, (dict, list, tuple, APIModel))):
-                response_data = json.dumps(response_data_raw, cls=APIJSONEncoder)
-            else:
-                response_data = response_data_raw
-            response_header = copyRequest.responseHeader
+            copyRequest = self.__request[definition]()
+            copyRequest.app = self
+            copyRequest._BaseAPIRequest__request = request
+            copyRequest._BaseAPIRequest__response = http_response()
+            for f in self.__fixedHeaders:
+                if f in request.headers:
+                    copyRequest._BaseAPIRequest__response.headers[request.headers._lowerKeys[f.lower()]] = request.headers[f]
+            copyRequest.client_address = addr[0]
+            
+            response_data = copyRequest.response.getResponse(copyRequest._BaseAPIRequest__methods[request.method if request.method != 'HEAD' else 'GET'](copyRequest, **vars))
+            response_code = copyRequest.response.response_code
+            response_status = RETURN_STATUS(response_code)
+            response_header = copyRequest.response.responseHeader
             
         except APIException as e:
             response_code = e.code
             response_status = RETURN_STATUS(response_code)
-            
-            response_data = json.dumps(e.response if e.response else RETURN_STATUS_OBJ(response_code))
-            response_header = e.responseHeader
+
+            response = http_response()
+            response.headers.update(e.header)
+            response_data = response.getResponse(e.response if e.response else RETURN_STATUS_OBJ(response_code))
+            response_header = response.responseHeader
         except Exception as e:
             response_code = INTERNAL_SERVER_ERROR
             response_status = RETURN_STATUS(INTERNAL_SERVER_ERROR)
-
-            response_data = json.dumps(RETURN_STATUS_OBJ(response_code))
-            response_header = 'content-type: {}\r\n'.format(JSON_UTF8)
+            
+            response = http_response()
+            response_data = response.getResponse(RETURN_STATUS_OBJ(response_code))
+            response_header = response.responseHeader
             print(e)
         
-        sock.sendall(response.format(response_status, len(response_data), response_header, response_data if request.method != 'HEAD' else '').encode())
-        response_time = round(time.time() - stime, 5)
-        print("{} - Request from {}: {} {} {}, {}{}\033[0m, {}s".format(datetime.now().strftime('%Y-%m-%d %H:%M:%S'), addr[0], request.method, request.uri, request.version, '\033[92m' if response_code == RESPONSE_OK else '\033[91m',RETURN_STATUS(response_code), response_time))
+        response_bytes = response_format.format(response_status, len(response_data), response_header).encode()
+        if(request.method != 'HEAD'):
+            response_bytes += response_data
+        sock.sendall(response_bytes)
+        response_time = round(time() - stime, 5)
+        print("{} - Request from {}: {} {} {}, {}{}\033[0m, {}s".format(datetime.now().strftime('%Y-%m-%d %H:%M:%S'), addr[0], request.method, request.uri, request.version, '\033[92m' if response_code >= 200 and response_code < 300 else '\033[91m' if response_code >=400 else '',RETURN_STATUS(response_code), response_time))
         sock.shutdown(socket.SHUT_WR)
         sock.close()
     def __handle_signal(self, signum, frame):
-        if signum == signal.SIGINT:
+        if signum == SIGINT:
             print('Received Ctrl+C signal')
             self.__running = False
-        if signum == signal.SIGTERM:
+        if signum == SIGTERM:
             print('Process Killed')
             self.__running = False
     def __init_socket(self):
         server = socket.socket()
         server.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-        server.setblocking(False)
+        server.settimeout(1)
         server.bind((self.__config['host'], self.__config['port']))
         server.listen(5)
-        signal.signal(signal.SIGINT, self.__handle_signal)
-        signal.signal(signal.SIGTERM, self.__handle_signal)
+        signal(SIGINT, self.__handle_signal)
+        signal(SIGTERM, self.__handle_signal)
         if 'ssl_certfile' in self.__config or 'ssl_keyfile' in self.__config:
             import ssl
             if 'ssl_certfile' not in self.__config:
@@ -211,3 +249,4 @@ class liteapi:
                 self.__socket = context.wrap_socket(server, server_side=True)
         else:
             self.__socket = server
+        self.__socket.settimeout(1)
